@@ -155,6 +155,50 @@ struct TMH_MerchantInstance
 };
 
 
+/**
+ * Event triggered when an order is paid.
+ */
+struct TMH_OrderPayEvent
+{
+  /**
+   * Type is #TALER_DBEVENT_MERCHANT_ORDER_PAID
+   */
+  struct GNUNET_DB_EventHeaderP header;
+
+  /**
+   * Always zero (for alignment).
+   */
+  uint32_t reserved;
+
+  /**
+   * Hash of the order ID.
+   */
+  struct GNUNET_HashCode h_order_id;
+};
+
+
+/**
+ * Event triggered when an order's refund is increased.
+ */
+struct TMH_OrderRefundEvent
+{
+  /**
+   * Type is #TALER_DBEVENT_MERCHANT_ORDER_REFUND
+   */
+  struct GNUNET_DB_EventHeaderP header;
+
+  /**
+   * Always zero (for alignment).
+   */
+  uint32_t reserved;
+
+  /**
+   * Hash of the order ID.
+   */
+  struct GNUNET_HashCode h_order_id;
+};
+
+
 /**
  * @brief Struct describing an URL and the handler for it.
  *
@@ -452,16 +496,6 @@ extern struct GNUNET_TIME_Relative TMH_legal_expiration;
 extern char *TMH_default_auth;
 
 
-/**
- * Kick MHD to run now, to be called after MHD_resume_connection().
- * Basically, we need to explicitly resume MHD's event loop whenever
- * we made progress serving a request.  This function re-schedules
- * the task processing MHD's activities to run immediately.
- */
-void
-TMH_trigger_daemon (void);
-
-
 /**
  * Suspend connection from @a sc until payment has been received.
  *
