@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2020 Taler Systems SA
+  (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -365,9 +365,11 @@ process_wire_fees (struct Exchange *exchange,
           (NULL != endp->next) )
     endp = endp->next;
   while ( (NULL != endp) &&
+          (NULL != fees) &&
           (fees->start_date.abs_value_us < endp->end_date.abs_value_us) )
     fees = fees->next;
   if ( (NULL != endp) &&
+       (NULL != fees) &&
        (fees->start_date.abs_value_us != endp->end_date.abs_value_us) )
   {
     /* Hole in the fee structure, not allowed! */
@@ -564,9 +566,11 @@ process_find_operations (struct Exchange *exchange)
       if (NULL == fbw)
       {
         need_wire = true;
-        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                    "Exchange does not support `%s' wire method (will retry later)\n",
-                    fo->wire_method);
+        /* Do not warn if this is before our first attempt */
+        if (! GNUNET_TIME_relative_is_zero (exchange->wire_retry_delay))
+          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                      "Exchange does not support `%s' wire method (will retry later)\n",
+                      fo->wire_method);
         fbw = NULL;
         continue;
       }
@@ -737,6 +741,62 @@ wire_task_cb (void *cls)
 }
 
 
+/**
+ * Free @a exchange.
+ *
+ * @param[in] exchange entry to free
+ */
+static void
+free_exchange_entry (struct Exchange *exchange)
+{
+  struct FeesByWireMethod *f;
+
+  GNUNET_CONTAINER_DLL_remove (exchange_head,
+                               exchange_tail,
+                               exchange);
+  while (NULL != (f = exchange->wire_fees_head))
+  {
+    struct TALER_EXCHANGE_WireAggregateFees *af;
+
+    GNUNET_CONTAINER_DLL_remove (exchange->wire_fees_head,
+                                 exchange->wire_fees_tail,
+                                 f);
+    while (NULL != (af = f->af))
+    {
+      f->af = af->next;
+      GNUNET_free (af);
+    }
+    GNUNET_free (f->wire_method);
+    GNUNET_free (f->payto_uri);
+    GNUNET_free (f);
+  }
+  if (NULL != exchange->wire_request)
+  {
+    TALER_EXCHANGE_wire_cancel (exchange->wire_request);
+    exchange->wire_request = NULL;
+  }
+  if (NULL != exchange->wire_task)
+  {
+    GNUNET_SCHEDULER_cancel (exchange->wire_task);
+    exchange->wire_task = NULL;
+  }
+  if (NULL != exchange->conn)
+  {
+    TALER_EXCHANGE_disconnect (exchange->conn);
+    exchange->conn = NULL;
+  }
+  if (NULL != exchange->retry_task)
+  {
+    GNUNET_SCHEDULER_cancel (exchange->retry_task);
+    exchange->retry_task = NULL;
+  }
+  GNUNET_assert (NULL == exchange->fo_head);
+  GNUNET_assert (NULL == exchange->fo_tail);
+  GNUNET_free (exchange->url);
+  GNUNET_free (exchange);
+}
+
+
 /**
  * We failed downloading /keys from @a exchange. Tell clients
  * about our failure, abort pending operations and retry later.
@@ -785,6 +845,13 @@ fail_and_retry (struct Exchange *exchange,
     exchange->retry_delay = GNUNET_TIME_relative_max (GNUNET_TIME_UNIT_HOURS,
                                                       exchange->retry_delay);
   }
+  if ( (NULL == exchange->fo_head) &&
+       (TALER_EC_GENERIC_CONFIGURATION_INVALID == hr->ec) )
+  {
+    /* This can NEVER work, so don't retry */
+    free_exchange_entry (exchange);
+    return;
+  }
   exchange->retry_delay = RETRY_BACKOFF (exchange->retry_delay);
   GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
               "Failed to fetch /keys from `%s': %d/%u, retrying in %s\n",
@@ -828,7 +895,8 @@ keys_mgmt_cb (void *cls,
   struct GNUNET_TIME_Absolute expire;
   struct GNUNET_TIME_Relative delay;
 
-  if (NULL == keys)
+  if ( (MHD_HTTP_OK != hr->http_status) ||
+       (NULL == keys) )
   {
     fail_and_retry (exchange,
                     hr,
@@ -846,8 +914,22 @@ keys_mgmt_cb (void *cls,
     exchange->trusted = false;
   }
   if (! exchange->trusted)
+  {
     exchange->master_pub = keys->master_pub;
-
+    for (struct Exchange *e = exchange_head;
+         NULL != e;
+         e = e->next)
+    {
+      if (e == exchange)
+        continue;
+      if (! e->trusted)
+        continue;
+      if (0 ==
+          GNUNET_memcmp (&e->master_pub,
+                         &exchange->master_pub))
+        exchange->trusted = true; /* same exchange, different URL => trust applies */
+    }
+  }
   if (0 != (TALER_EXCHANGE_VC_NEWER & compat))
   {
     /* Warn user exactly once about need to upgrade */
@@ -894,6 +976,12 @@ keys_mgmt_cb (void *cls,
     delay = RELOAD_DELAY;
   else
     delay = GNUNET_TIME_absolute_get_remaining (expire);
+  if (GNUNET_TIME_relative_is_zero (delay))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "/keys response from exchange expired immediately! Retrying in 1 minute.\n");
+    delay = GNUNET_TIME_UNIT_MINUTES;
+  }
   exchange->retry_delay = GNUNET_TIME_UNIT_ZERO;
   if (NULL != exchange->retry_task)
     GNUNET_SCHEDULER_cancel (exchange->retry_task);
@@ -1001,11 +1089,8 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
   GNUNET_CONTAINER_DLL_insert (exchange->fo_head,
                                exchange->fo_tail,
                                fo);
-  now = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&now);
   if ( (force_reload) &&
-       (0 == GNUNET_TIME_absolute_get_remaining (
-          exchange->first_retry).rel_value_us) )
+       (GNUNET_TIME_absolute_is_past (exchange->first_retry)) )
   {
     /* increment exponential-backoff */
     exchange->retry_delay = RETRY_BACKOFF (exchange->retry_delay);
@@ -1020,7 +1105,8 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
     return fo;
   }
 
-
+  now = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&now);
   if ( (! exchange->pending) &&
        ( (NULL == fo->wire_method) ||
          (NULL != get_wire_fees (exchange,
@@ -1200,10 +1286,11 @@ TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
 
     if (! exchange->trusted)
       continue;
-    j_exchange = json_pack ("{s:s, s:o}",
-                            "url", exchange->url,
-                            "master_pub", GNUNET_JSON_from_data_auto (
-                              &exchange->master_pub));
+    j_exchange = GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_string ("url",
+                               exchange->url),
+      GNUNET_JSON_pack_data_auto ("master_pub",
+                                  &exchange->master_pub));
     GNUNET_assert (0 ==
                    json_array_append_new (TMH_trusted_exchanges,
                                           j_exchange));
@@ -1215,56 +1302,8 @@ TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
 void
 TMH_EXCHANGES_done ()
 {
-  struct Exchange *exchange;
-
-  while (NULL != (exchange = exchange_head))
-  {
-    struct FeesByWireMethod *f;
-
-    GNUNET_CONTAINER_DLL_remove (exchange_head,
-                                 exchange_tail,
-                                 exchange);
-    while (NULL != (f = exchange->wire_fees_head))
-    {
-      struct TALER_EXCHANGE_WireAggregateFees *af;
-
-      GNUNET_CONTAINER_DLL_remove (exchange->wire_fees_head,
-                                   exchange->wire_fees_tail,
-                                   f);
-      while (NULL != (af = f->af))
-      {
-        f->af = af->next;
-        GNUNET_free (af);
-      }
-      GNUNET_free (f->wire_method);
-      GNUNET_free (f->payto_uri);
-      GNUNET_free (f);
-    }
-    if (NULL != exchange->wire_request)
-    {
-      TALER_EXCHANGE_wire_cancel (exchange->wire_request);
-      exchange->wire_request = NULL;
-    }
-    if (NULL != exchange->wire_task)
-    {
-      GNUNET_SCHEDULER_cancel (exchange->wire_task);
-      exchange->wire_task = NULL;
-    }
-    if (NULL != exchange->conn)
-    {
-      TALER_EXCHANGE_disconnect (exchange->conn);
-      exchange->conn = NULL;
-    }
-    if (NULL != exchange->retry_task)
-    {
-      GNUNET_SCHEDULER_cancel (exchange->retry_task);
-      exchange->retry_task = NULL;
-    }
-    GNUNET_assert (NULL == exchange->fo_head);
-    GNUNET_assert (NULL == exchange->fo_tail);
-    GNUNET_free (exchange->url);
-    GNUNET_free (exchange);
-  }
+  while (NULL != exchange_head)
+    free_exchange_entry (exchange_head);
   GNUNET_CURL_fini (merchant_curl_ctx);
   merchant_curl_ctx = NULL;
   GNUNET_CURL_gnunet_rc_destroy (merchant_curl_rc);
