@@ -23,6 +23,7 @@
 #include <jansson.h>
 #include <gnunet/gnunet_uri_lib.h>
 #include <taler/taler_signatures.h>
+#include <taler/taler_dbevents.h>
 #include <taler/taler_json_lib.h>
 #include <taler/taler_exchange_service.h>
 #include "taler-merchant-httpd_exchanges.h"
@@ -74,6 +75,11 @@ struct GetOrderData
    */
   struct TMH_SuspendedConnection sc;
 
+  /**
+   * Database event we are waiting on to be resuming.
+   */
+  struct GNUNET_DB_EventHandler *eh;
+
   /**
    * Which merchant instance is this for?
    */
@@ -175,10 +181,39 @@ TMH_force_wallet_get_order_resume (void)
     GNUNET_assert (god->suspended);
     god->suspended = false;
     MHD_resume_connection (god->sc.con);
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
 }
 
 
+/**
+ * We have received a trigger from the database
+ * that we should (possibly) resume the request.
+ *
+ * @param cls a `struct GetOrderData` to resume
+ * @param extra string encoding refund amount (or NULL)
+ * @param extra_size number of bytes in @a extra
+ */
+static void
+resume_by_event (void *cls,
+                 const void *extra,
+                 size_t extra_size)
+{
+  struct GetOrderData *god = cls;
+
+  (void) extra;
+  (void) extra_size;
+  if (! god->suspended)
+    return; /* duplicate event is possible */
+  god->suspended = false;
+  GNUNET_CONTAINER_DLL_insert (god_head,
+                               god_tail,
+                               god);
+  MHD_resume_connection (god->sc.con);
+  TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
+}
+
+
 /**
  * Suspend this @a god until the trigger is satisfied.
  *
@@ -691,6 +726,11 @@ god_cleanup (void *cls)
     json_decref (god->contract_terms);
     god->contract_terms = NULL;
   }
+  if (NULL != god->eh)
+  {
+    TMH_db->event_listen_cancel (god->eh);
+    god->eh = NULL;
+  }
   GNUNET_free (god);
 }
 
@@ -770,37 +810,8 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
     god->session_id = MHD_lookup_connection_value (connection,
                                                    MHD_GET_ARGUMENT_KIND,
                                                    "session_id");
-    {
-      const char *long_poll_timeout_ms;
-
-      long_poll_timeout_ms = MHD_lookup_connection_value (connection,
-                                                          MHD_GET_ARGUMENT_KIND,
-                                                          "timeout_ms");
-      if (NULL != long_poll_timeout_ms)
-      {
-        unsigned int timeout;
-        char dummy;
-
-        if (1 != sscanf (long_poll_timeout_ms,
-                         "%u%c",
-                         &timeout,
-                         &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms (must be non-negative number)");
-        }
-        /* If HTML is requested, we never long poll. Makes no sense */
-        if (! god->generate_html)
-          god->sc.long_poll_timeout
-            = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_multiply (
-                                                  GNUNET_TIME_UNIT_MILLISECONDS,
-                                                  timeout));
-      }
-    }
 
+    /* process await_refund_obtained argument */
     {
       const char *await_refund_obtained_s;
 
@@ -810,7 +821,8 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                      "await_refund_obtained");
       god->sc.awaiting_refund_obtained =
         (NULL != await_refund_obtained_s)
-        ? 0 == strcasecmp (await_refund_obtained_s, "yes")
+        ? 0 == strcasecmp (await_refund_obtained_s,
+                           "yes")
         : false;
     }
 
@@ -837,6 +849,80 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
         god->sc.awaiting_refund = true;
       }
     }
+
+
+    /* process timeout_ms argument */
+    {
+      const char *long_poll_timeout_ms;
+
+      long_poll_timeout_ms = MHD_lookup_connection_value (connection,
+                                                          MHD_GET_ARGUMENT_KIND,
+                                                          "timeout_ms");
+      if (NULL != long_poll_timeout_ms)
+      {
+        unsigned int timeout_ms;
+        char dummy;
+
+        if (1 != sscanf (long_poll_timeout_ms,
+                         "%u%c",
+                         &timeout_ms,
+                         &dummy))
+        {
+          GNUNET_break_op (0);
+          return TALER_MHD_reply_with_error (connection,
+                                             MHD_HTTP_BAD_REQUEST,
+                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                             "timeout_ms (must be non-negative number)");
+        }
+        /* If HTML is requested, we never long poll. Makes no sense */
+        if (! god->generate_html)
+        {
+          struct GNUNET_TIME_Relative timeout;
+
+          timeout = GNUNET_TIME_relative_multiply (
+            GNUNET_TIME_UNIT_MILLISECONDS,
+            timeout_ms);
+          god->sc.long_poll_timeout
+            = GNUNET_TIME_relative_to_absolute (timeout);
+          if (! GNUNET_TIME_relative_is_zero (timeout))
+          {
+            if (god->sc.awaiting_refund)
+            {
+              struct TMH_OrderPayEvent refund_eh = {
+                .header.size = htons (sizeof (refund_eh)),
+                .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_REFUND)
+              };
+
+              GNUNET_CRYPTO_hash (god->order_id,
+                                  strlen (god->order_id),
+                                  &refund_eh.h_order_id);
+              god->eh = TMH_db->event_listen (TMH_db->cls,
+                                              &refund_eh.header,
+                                              timeout,
+                                              &resume_by_event,
+                                              god);
+            }
+            else /* ! waiting for refund */
+            {
+              struct TMH_OrderPayEvent pay_eh = {
+                .header.size = htons (sizeof (pay_eh)),
+                .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID)
+              };
+
+              GNUNET_CRYPTO_hash (god->order_id,
+                                  strlen (god->order_id),
+                                  &pay_eh.h_order_id);
+              god->eh = TMH_db->event_listen (TMH_db->cls,
+                                              &pay_eh.header,
+                                              timeout,
+                                              &resume_by_event,
+                                              god);
+            }
+          } /* end of timeout non-zero */
+        } /* end of HTML generation NOT requested */
+      } /* end of timeout_ms argument provided */
+    } /* end of timeout_ms argument handling */
+
   } /* end of first-time initialization / sanity checks */
 
   if (god->suspended)
