@@ -310,7 +310,7 @@ resume_abort_with_response (struct AbortContext *ac,
   GNUNET_assert (GNUNET_YES == ac->suspended);
   ac->suspended = GNUNET_NO;
   MHD_resume_connection (ac->connection);
-  TMH_trigger_daemon (); /* we resumed, kick MHD */
+  TALER_MHD_daemon_trigger (); /* we resumed, kick MHD */
 }
 
 
