@@ -469,7 +469,7 @@ resume_pay_with_response (struct PayContext *pc,
   GNUNET_assert (GNUNET_YES == pc->suspended);
   pc->suspended = GNUNET_NO;
   MHD_resume_connection (pc->connection);
-  TMH_trigger_daemon (); /* we resumed, kick MHD */
+  TALER_MHD_daemon_trigger (); /* we resumed, kick MHD */
 }
 
 
