@@ -353,7 +353,7 @@ check_resume_prd (struct PostRefundData *prd)
   GNUNET_assert (prd->suspended);
   prd->suspended = false;
   MHD_resume_connection (prd->sc.con);
-  TMH_trigger_daemon ();
+  TALER_MHD_daemon_trigger ();
 }
 
 
