@@ -297,7 +297,7 @@ withdraw_cb (void *cls,
         TALER_JSON_pack_ec (TALER_EC_MERCHANT_TIP_PICKUP_EXCHANGE_ERROR),
         TMH_pack_exchange_reply (hr));
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   qs = TMH_db->insert_pickup_blind_signature (TMH_db->cls,
@@ -313,14 +313,14 @@ withdraw_cb (void *cls,
       TALER_EC_GENERIC_DB_STORE_FAILED,
       "blind signature");
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   if (NULL == pc->po_head)
   {
     stop_operations (pc); /* stops timeout job */
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
 }
 
@@ -361,7 +361,7 @@ do_withdraw (void *cls,
       TALER_JSON_pack_ec (
         TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT));
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   if (NULL == eh)
@@ -378,7 +378,7 @@ do_withdraw (void *cls,
           TALER_EC_MERCHANT_GENERIC_EXCHANGE_CONNECT_FAILURE),
         TMH_pack_exchange_reply (hr));
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   po->w2h = TALER_EXCHANGE_withdraw2 (eh,
@@ -442,7 +442,7 @@ do_timeout (void *cls)
     TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT,
     NULL);
   MHD_resume_connection (pc->connection);
-  TMH_trigger_daemon ();   /* we resumed, kick MHD */
+  TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
 }
 
 
@@ -479,7 +479,7 @@ compute_total_requested (void *cls,
       TALER_JSON_pack_ec (
         TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT));
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   if (NULL == eh)
@@ -491,7 +491,7 @@ compute_total_requested (void *cls,
           TALER_EC_MERCHANT_GENERIC_EXCHANGE_CONNECT_FAILURE),
         TMH_pack_exchange_reply (hr));
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   if (NULL == (keys = TALER_EXCHANGE_get_keys (eh)))
@@ -503,7 +503,7 @@ compute_total_requested (void *cls,
           TALER_EC_MERCHANT_GENERIC_EXCHANGE_KEYS_FAILURE),
         TMH_pack_exchange_reply (hr));
     MHD_resume_connection (pc->connection);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
   }
   GNUNET_assert (GNUNET_OK ==
@@ -525,7 +525,7 @@ compute_total_requested (void *cls,
             TALER_EC_MERCHANT_TIP_PICKUP_DENOMINATION_UNKNOWN),
           TMH_pack_exchange_reply (hr));
       MHD_resume_connection (pc->connection);
-      TMH_trigger_daemon ();   /* we resumed, kick MHD */
+      TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
       return;
     }
 
@@ -542,13 +542,13 @@ compute_total_requested (void *cls,
         TALER_MHD_make_error (TALER_EC_MERCHANT_TIP_PICKUP_SUMMATION_FAILED,
                               "Could not add up values to compute pickup total");
       MHD_resume_connection (pc->connection);
-      TMH_trigger_daemon ();   /* we resumed, kick MHD */
+      TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
       return;
     }
   }
   pc->tr_initialized = true;
   MHD_resume_connection (pc->connection);
-  TMH_trigger_daemon ();   /* we resumed, kick MHD */
+  TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
 }
 
 
