@@ -177,7 +177,7 @@ order_timeout (void *cls)
                                  po);
     po->in_dll = false;
     MHD_resume_connection (po->con);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
   order_timeout_task = GNUNET_SCHEDULER_add_at (po->long_poll_timeout,
                                                 &order_timeout,
@@ -452,7 +452,7 @@ TMH_notify_order_change (struct TMH_MerchantInstance *mi,
     GNUNET_assert (po ==
                    GNUNET_CONTAINER_heap_remove_node (po->hn));
     MHD_resume_connection (po->con);
-    TMH_trigger_daemon ();   /* we resumed, kick MHD */
+    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
 }
 
