@@ -215,7 +215,7 @@ resume_transfer_with_response (struct PostTransfersContext *ptc,
                                ptc);
   ptc->suspended = false;
   MHD_resume_connection (ptc->connection);
-  TMH_trigger_daemon ();   /* we resumed, kick MHD */
+  TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
 }
 
 
