@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2018 Taler Systems SA
+  (C) 2018-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -120,6 +120,25 @@ TALER_TESTING_cmd_merchant_post_instances (const char *label,
                                            unsigned int http_status);
 
 
+/**
+ * Define a "POST /private/auth" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /instances request.
+ * @param instance_id the ID of the instance, or NULL
+ * @param auth_token new auth token to use, can be NULL for no auth
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_instance_auth (const char *label,
+                                               const char *merchant_url,
+                                               const char *instance_id,
+                                               const char *auth_token,
+                                               unsigned int http_status);
+
+
 /**
  * Define a "POST /instances" CMD.  Comprehensive version.
  *
@@ -137,6 +156,7 @@ TALER_TESTING_cmd_merchant_post_instances (const char *label,
  * @param default_max_deposit_fee default maximum deposit fee merchant is willing to pay
  * @param default_wire_transfer_delay default wire transfer delay merchant will ask for
  * @param default_pay_delay default validity period for offers merchant makes
+ * @param auth_token authorization token needed to access the instance, can be NULL
  * @param http_status expected HTTP response code.
  * @return the command.
  */
@@ -155,6 +175,7 @@ TALER_TESTING_cmd_merchant_post_instances2 (
   const char *default_max_deposit_fee,
   struct GNUNET_TIME_Relative default_wire_transfer_delay,
   struct GNUNET_TIME_Relative default_pay_delay,
+  const char *auth_token,
   unsigned int http_status);
 
 
@@ -314,7 +335,7 @@ TALER_TESTING_cmd_merchant_post_products2 (
   json_t *description_i18n,
   const char *unit,
   const char *price,
-  json_t *image,
+  const char *image,
   json_t *taxes,
   int64_t total_stock,
   json_t *address,
@@ -378,7 +399,7 @@ TALER_TESTING_cmd_merchant_patch_product (
   json_t *description_i18n,
   const char *unit,
   const char *price,
-  json_t *image,
+  const char *image,
   json_t *taxes,
   int64_t total_stock,
   uint64_t total_lost,
@@ -538,20 +559,19 @@ TALER_TESTING_cmd_merchant_post_orders_no_claim (const char *label,
  * @return the command
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_post_orders2 (const char *label,
-                                         const char *merchant_url,
-                                         unsigned int http_status,
-                                         const char *order_id,
-                                         struct GNUNET_TIME_Absolute
-                                         refund_deadline,
-                                         struct GNUNET_TIME_Absolute
-                                         pay_deadline,
-                                         bool claim_token,
-                                         const char *amount,
-                                         const char *payment_target,
-                                         const char *products,
-                                         const char *locks,
-                                         const char *duplicate_of);
+TALER_TESTING_cmd_merchant_post_orders2 (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *order_id,
+  struct GNUNET_TIME_Absolute refund_deadline,
+  struct GNUNET_TIME_Absolute pay_deadline,
+  bool claim_token,
+  const char *amount,
+  const char *payment_target,
+  const char *products,
+  const char *locks,
+  const char *duplicate_of);
 
 
 /**
@@ -976,6 +996,43 @@ TALER_TESTING_cmd_merchant_post_transfer (
   ...);
 
 
+/**
+ * Define a POST /transfers CMD.  Details like the WTID and
+ * other required parameters are explicitly provided.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the backend
+ * @param payto_uri URL of the exchange's bank account
+ * @param credit_amount amount credited
+ * @param wtid wire transfer ID to use, NULL to pick one at random
+ * @param exchange_url URL of the exchange to use
+ * @param http_code expected HTTP response code
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_transfer2 (
+  const char *label,
+  const char *merchant_url,
+  const char *payto_uri,
+  const char *credit_amount,
+  const char *wtid,
+  const char *exchange_url,
+  unsigned int http_code);
+
+
+/**
+ * We have discovered the @a serial for a POST transfer
+ * command. Store it (for the bank row trait).
+ *
+ * @param cmd command to update
+ * @param serial serial number to store (identifies the row)
+ */
+void
+TALER_TESTING_cmd_merchant_post_transfer_set_serial (
+  struct TALER_TESTING_Command *cmd,
+  uint64_t serial);
+
+
 /**
  * Define a GET /transfers CMD.
  *
@@ -997,6 +1054,24 @@ TALER_TESTING_cmd_merchant_get_transfers (const char *label,
                                           ...);
 
 
+/**
+ * Define a "DELETE transfer" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant for the
+ *        DELETE /transfers/$ID request.
+ * @param transfer_ref reference to a command that
+ *             yields the transfer ID to delete
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_delete_transfer (const char *label,
+                                            const char *merchant_url,
+                                            const char *transfer_ref,
+                                            unsigned int http_status);
+
+
 /* ******************* /reserves *************** */
 
 
@@ -1325,15 +1400,15 @@ TALER_TESTING_cmd_tip_authorize_with_ec (const char *label,
  * @param ec expected Taler-defined error code.
  */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_tip_authorize_from_reserve_with_ec (const char *label,
-                                                      const char *merchant_url,
-                                                      const char *exchange_url,
-                                                      const char *
-                                                      reserve_reference,
-                                                      unsigned int http_status,
-                                                      const char *justification,
-                                                      const char *amount,
-                                                      enum TALER_ErrorCode ec);
+TALER_TESTING_cmd_tip_authorize_from_reserve_with_ec (
+  const char *label,
+  const char *merchant_url,
+  const char *exchange_url,
+  const char *reserve_reference,
+  unsigned int http_status,
+  const char *justification,
+  const char *amount,
+  enum TALER_ErrorCode ec);
 
 
 /**
@@ -1433,9 +1508,9 @@ TALER_TESTING_get_trait_merchant_sig (
  * @return the trait
  */
 struct TALER_TESTING_Trait
-TALER_TESTING_make_trait_claim_nonce (unsigned int index,
-                                      const struct
-                                      GNUNET_CRYPTO_EddsaPublicKey *nonce);
+TALER_TESTING_make_trait_claim_nonce (
+  unsigned int index,
+  const struct GNUNET_CRYPTO_EddsaPublicKey *nonce);
 
 
 /**
@@ -1445,14 +1520,13 @@ TALER_TESTING_make_trait_claim_nonce (unsigned int index,
  * @param index which nonce to pick if @a
  *        cmd has multiple on offer
  * @param[out] nonce set to the wanted data.
- *
  * @return #GNUNET_OK on success
  */
 int
-TALER_TESTING_get_trait_claim_nonce (const struct TALER_TESTING_Command *cmd,
-                                     unsigned int index,
-                                     const struct
-                                     GNUNET_CRYPTO_EddsaPublicKey **nonce);
+TALER_TESTING_get_trait_claim_nonce (
+  const struct TALER_TESTING_Command *cmd,
+  unsigned int index,
+  const struct GNUNET_CRYPTO_EddsaPublicKey **nonce);
 
 
 /**
@@ -1538,6 +1612,7 @@ TALER_TESTING_get_trait_planchet_secrets (
   unsigned int index,
   struct TALER_PlanchetSecretsP **planchet_secrets);
 
+
 /**
  * Offer planchet secrets.
  *
@@ -1550,6 +1625,7 @@ TALER_TESTING_make_trait_planchet_secrets (
   unsigned int index,
   const struct TALER_PlanchetSecretsP *planchet_secrets);
 
+
 /**
  * Offer tip id.
  *
@@ -1562,6 +1638,7 @@ struct TALER_TESTING_Trait
 TALER_TESTING_make_trait_tip_id (unsigned int index,
                                  const struct GNUNET_HashCode *tip_id);
 
+
 /**
  * Obtain tip id from a @a cmd.
  *
