@@ -721,6 +721,7 @@ struct TALER_MERCHANTDB_Plugin
    *
    * @param cls database context to use
    * @param es specification of the event to listen for
+   * @param timeout how long to wait for the event
    * @param cb function to call when the event happens, possibly
    *         multiple times (until cancel is invoked)
    * @param cb_cls closure for @a cb
@@ -729,6 +730,7 @@ struct TALER_MERCHANTDB_Plugin
   struct GNUNET_DB_EventHandler *
   (*event_listen)(void *cls,
                   const struct GNUNET_DB_EventHeaderP *es,
+                  struct GNUNET_TIME_Relative timeout,
                   GNUNET_DB_EventCallback cb,
                   void *cb_cls);
 
