@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2020 Taler Systems SA
+  Copyright (C) 2020-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
@@ -109,6 +109,11 @@ handle_post_products_finished (void *cls,
      * or the merchant is buggy (or API version conflict);
      * just pass JSON reply to the application */
     break;
+  case MHD_HTTP_UNAUTHORIZED:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    /* Nothing really to verify, merchant says we need to authenticate. */
+    break;
   case MHD_HTTP_FORBIDDEN:
     hr.ec = TALER_JSON_get_error_code (json);
     hr.hint = TALER_JSON_get_error_hint (json);
@@ -160,7 +165,7 @@ TALER_MERCHANT_products_post (
   const json_t *description_i18n,
   const char *unit,
   const struct TALER_Amount *price,
-  const json_t *image,
+  const char *image,
   const json_t *taxes,
   int64_t total_stock,
   const json_t *address,
@@ -172,34 +177,28 @@ TALER_MERCHANT_products_post (
   json_t *req_obj;
 
   (void) GNUNET_TIME_round_abs (&next_restock);
-  req_obj = json_pack ("{s:s, s:s, s:O, s:s, s:o,"
-                       " s:O, s:O, s:I, s:O, s:o}",
-                       "product_id",
-                       product_id,
-                       "description",
-                       description,
-                       "description_i18n",
-                       description_i18n,
-                       "unit",
-                       unit,
-                       "price",
-                       TALER_JSON_from_amount (price),
-                       /* End of first group of 5 */
-                       "image",
-                       image,
-                       "taxes",
-                       taxes,
-                       "total_stock",
-                       (json_int_t) total_stock,
-                       "address",
-                       address,
-                       "next_restock",
-                       GNUNET_JSON_from_time_abs (next_restock));
-  if (NULL == req_obj)
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
+  req_obj = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_string ("product_id",
+                             product_id),
+    GNUNET_JSON_pack_string ("description",
+                             description),
+    GNUNET_JSON_pack_object_incref ("description_i18n",
+                                    (json_t *) description_i18n),
+    GNUNET_JSON_pack_string ("unit",
+                             unit),
+    TALER_JSON_pack_amount ("price",
+                            price),
+    GNUNET_JSON_pack_string ("image",
+                             image),
+    GNUNET_JSON_pack_array_incref ("taxes",
+                                   (json_t *) taxes),
+    GNUNET_JSON_pack_uint64 ("total_stock",
+                             total_stock),
+    GNUNET_JSON_pack_object_incref ("address",
+                                    (json_t *) address),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_time_abs ("next_restock",
+                                 next_restock)));
   pph = GNUNET_new (struct TALER_MERCHANT_ProductsPostHandle);
   pph->ctx = ctx;
   pph->cb = cb;
