@@ -153,6 +153,9 @@ get_transfers_cb (
           TALER_TESTING_interpreter_fail (gts->is);
           return;
         }
+        TALER_TESTING_cmd_merchant_post_transfer_set_serial (
+          (struct TALER_TESTING_Command *) transfer_cmd,
+          transfers[i].credit_serial);
       }
       {
         const char *payto_uri;
